(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('rxjs/Subject'), require('rxjs/operator/share')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common', 'rxjs/Subject', 'rxjs/operator/share'], factory) :
	(factory((global.core = {}),global.ng.core,global.ng.common,global.Rx,global.Rx.Observable.prototype));
}(this, (function (exports,core,common,Subject,share) { 'use strict';

var LoadingBarService = /** @class */ (function () {
    function LoadingBarService() {
        this.pending = new Subject.Subject();
        this._pendingRequests = 0;
    }
    /**
     * @param {?=} source$
     * @return {?}
     */
    LoadingBarService.prototype.startLoading = function (source$) {
        this.pending.next({
            started: this._pendingRequests === 0,
            pendingRequests: ++this._pendingRequests,
        });
        if (source$) {
            share.share.call(source$).subscribe(this.getLoadingObserver());
        }
    };
    /**
     * @return {?}
     */
    LoadingBarService.prototype.endLoading = function () {
        if (this._pendingRequests === 0) {
            return;
        }
        this.pending.next({
            completed: this._pendingRequests === 1,
            pendingRequests: --this._pendingRequests,
        });
    };
    /**
     * @return {?}
     */
    LoadingBarService.prototype.getLoadingObserver = function () {
        var _this = this;
        return {
            next: function (x) { return x; },
            error: function (err) { return _this.endLoading(); },
            complete: function () { return _this.endLoading(); },
        };
    };
    /**
     * @return {?}
     */
    LoadingBarService.prototype.ngOnDestroy = function () {
        this.pending.complete();
    };
    return LoadingBarService;
}());
LoadingBarService.decorators = [
    { type: core.Injectable },
];
/**
 * @nocollapse
 */
LoadingBarService.ctorParameters = function () { return []; };
var LoadingBarComponent = /** @class */ (function () {
    /**
     * @param {?} loadingBarService
     */
    function LoadingBarComponent(loadingBarService) {
        var _this = this;
        this.includeSpinner = true;
        this.started = false;
        this.progressWidth = '0';
        this._latencyThreshold = 10;
        this._startSize = 0.02;
        this._status = 0;
        loadingBarService.pending.subscribe(function (progress) {
            if (progress.started)
                _this.start();
            if (progress.completed)
                _this.complete();
        });
    }
    /**
     * Inserts the loading bar element into the dom, and sets it to 2%
     * @return {?}
     */
    LoadingBarComponent.prototype.start = function () {
        var _this = this;
        this._startTimeout = setTimeout(function () {
            clearTimeout(_this._completeTimeout);
            // do not continually broadcast the started event:
            if (_this.started) {
                return;
            }
            _this.started = true;
            _this._status = 0;
            _this.set(_this._startSize);
        }, this._latencyThreshold);
    };
    /**
     * Set the loading bar's width to a certain percent.
     *
     * @param {?} n any value between 0 and 1
     * @return {?}
     */
    LoadingBarComponent.prototype.set = function (n) {
        var _this = this;
        if (!this.started) {
            return;
        }
        var /** @type {?} */ pct = (n * 100) + '%';
        this.progressWidth = pct;
        this._status = n;
        // increment loadingbar to give the illusion that there is always
        // progress but make sure to cancel the previous timeouts so we don't
        // have multiple incs running at the same time.
        clearTimeout(this._incTimeout);
        this._incTimeout = setTimeout(function () { return _this.inc(); }, 250);
    };
    /**
     * @return {?}
     */
    LoadingBarComponent.prototype.complete = function () {
        var _this = this;
        setTimeout(function () {
            _this.set(1);
            clearTimeout(_this._completeTimeout);
            clearTimeout(_this._startTimeout);
            // Attempt to aggregate any start/complete calls within 500ms:
            _this._completeTimeout = setTimeout(function () { return _this.started = false; }, 500);
        }, this._latencyThreshold);
    };
    /**
     * Increments the loading bar by a random amount
     * but slows down as it progresses
     * @return {?}
     */
    LoadingBarComponent.prototype.inc = function () {
        if (this._status >= 1) {
            return;
        }
        var /** @type {?} */ rnd = 0;
        var /** @type {?} */ stat = this._status;
        if (stat >= 0 && stat < 0.25) {
            // Start out between 3 - 6% increments
            rnd = (Math.random() * (5 - 3 + 1) + 3) / 100;
        }
        else if (stat >= 0.25 && stat < 0.65) {
            // increment between 0 - 3%
            rnd = (Math.random() * 3) / 100;
        }
        else if (stat >= 0.65 && stat < 0.9) {
            // increment between 0 - 2%
            rnd = (Math.random() * 2) / 100;
        }
        else if (stat >= 0.9 && stat < 0.99) {
            // finally, increment it .5 %
            rnd = 0.005;
        }
        else {
            // after 99%, don't increment:
            rnd = 0;
        }
        this.set(this._status + rnd);
    };
    return LoadingBarComponent;
}());
LoadingBarComponent.decorators = [
    { type: core.Component, args: [{
                selector: 'ng-loading-bar, ngx-loading-bar',
                template: "\n    <ng-container *ngIf=\"started\">\n        <div id=\"loading-bar-spinner\" *ngIf=\"includeSpinner\"><div class=\"spinner-icon\"></div></div>\n        <div id=\"loading-bar\"><div class=\"bar\" [style.width]=\"progressWidth\"><div class=\"peg\"></div></div></div>\n    </ng-container>\n    ",
            },] },
];
/**
 * @nocollapse
 */
LoadingBarComponent.ctorParameters = function () { return [
    { type: LoadingBarService, },
]; };
LoadingBarComponent.propDecorators = {
    'includeSpinner': [{ type: core.Input },],
};
var LoadingBarModule = /** @class */ (function () {
    function LoadingBarModule() {
    }
    /**
     * @return {?}
     */
    LoadingBarModule.forRoot = function () {
        return {
            ngModule: LoadingBarModule,
            providers: [LoadingBarService],
        };
    };
    return LoadingBarModule;
}());
LoadingBarModule.decorators = [
    { type: core.NgModule, args: [{
                imports: [common.CommonModule],
                declarations: [LoadingBarComponent],
                exports: [LoadingBarComponent],
            },] },
];
/**
 * @nocollapse
 */
LoadingBarModule.ctorParameters = function () { return []; };

exports.LoadingBarModule = LoadingBarModule;
exports.LoadingBarService = LoadingBarService;
exports.LoadingBarComponent = LoadingBarComponent;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=core.umd.js.map
